"""
Copyright start
Copyright (C) 2008 - 2024 FortinetInc.
All rights reserved.
FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
Copyright end
"""

import socketserver
import logging
import os
from datetime import datetime
import sys
from os import path
sys.path.append(path.abspath('/opt/cyops-integrations/integrations'))
from integrations.crudhub import make_request

LOG_DIR_PATH = '/var/log/cyops/cyops-integrations/syslog/'
LOG_FILE_PATH = path.join(LOG_DIR_PATH, 'listener.log')
os.makedirs(LOG_DIR_PATH, exist_ok=True)

logging.basicConfig(filename=LOG_FILE_PATH,level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(module)s %(funcName)s(): %(message)s')
cs_host = 'localhost'
DEFAULT_ALGORITHM = 'sha256'
timestamp = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")

logger = logging.getLogger(__name__)
SSL_VERIFY = False

class SyslogHandler(socketserver.BaseRequestHandler):

    def handle(self):
        logger.error('got new message')
        if self.server.transport and self.server.transport.lower() == 'tcp':
            data = self.request.recv(1024).decode('utf-8')
        else:
            #assuming udp by default
            data = bytes.decode(self.request[0].strip())
        logger.info("%s : %s" % (self.client_address[0], str(data)))
        logger.info('filter string is :%s' % self.server.filter_str)
        if (not self.server.filter_str) or (self.server.filter_str.strip() == '') or (self.server.filter_str in data):
            logger.info('Triggering playbook for message.')
            self.forward_to_cyops(data)
        else:
            logger.info('Message did not match the filter string. Not triggering playbook.')

    def forward_to_cyops(self, data):
        if data:
            # Replace if any unicode sequences in data
            data = data.replace("\u0000",'')
            full_uri = 'https://' + cs_host + '/api/triggers/1/' + self.server.trigger
            try:
                response = make_request(full_uri, 'POST', body=data, __async=True, verify=False)
                if response.get('task_id'):
                    logger.info('Playbook for fetching mail triggered successfully')
                else:
                    logger.error('Error invoking playbook')
            except Exception as e:
                logger.exception(
                    "Error occurred while triggering the CyOPs playbook ' {0} ' ERROR :: {1}".format(self.server.trigger, e))
                raise Exception(
                    "Error occurred while triggering the CyOPs playbook ' {0} ' ERROR :: {1}".format(self.server.trigger, e))